(function ($) {
  Drupal.cacheControl = {};
  Drupal.cacheControl.initialized = false;

  Drupal.behaviors.cacheControl = {
    attach: function(context) {
      if (Drupal.cacheControl.initialized) {
        //There's no need to ever execute this more than once per pageload
        cache_control_debug('cc already initialized');
        return;
      }
      Drupal.cacheControl.initialized = true;
      cache_control_debug('cc active');
      if (Drupal.settings.cacheControl && (Drupal.settings.cacheControl.cachePageComponentIds || Drupal.settings.cacheControl.cacheNodeId)) {
        cache_control_debug('cc running');
        var cachePageId = Drupal.settings.cacheControl.cachePageId;
        var cacheNodeId = Drupal.settings.cacheControl.cacheNodeId;

        var force = Drupal.settings.cacheControl.force || false;
        var authenticated = parseInt($.cookie('cacheControlAuthenticated'));
        cache_control_debug(parseInt($.cookie('cacheControlAuthenticated')));
        if (authenticated || force) {
          cache_control_debug('cc getting components');
          var path = Drupal.settings.basePath
          + "_cache_control/get_components/u/"
          + $.cookie('cacheControlAuthenticated')
          + "/"
          + $.cookie('cacheControlSessionHash')
          + "/"
          + Drupal.settings.cacheControl.cachePageComponentIds;

          $.getJSON(path, function(data) {
            //Inject components into DOM
            cache_control_debug(data);
            for (i in data) {
              cache_control_debug('cc handling element ' + i);
              var elem = $(data[i]);
              cache_control_debug(elem);
              var id = elem.attr("id");
              cache_control_debug(id);
              //Remove the anonymous content to avoid conflicts
              var anonymousElement = $("#"+id+"-anonymous");
              var placeholderElement = $("#"+(id));
              Drupal.detachBehaviors(anonymousElement);

              if (placeholderElement.length > 0) {
            	anonymousElement.remove();
                Drupal.detachBehaviors(placeholderElement);
                placeholderElement.replaceWith(elem);
              }
              else {
            	anonymousElement.replaceWith(elem);
              }

              //Some JS components may need initializing
              for (b in Drupal.behaviors) {
                if (b != 'cacheControl' && typeof Drupal.behaviors[b].attach == 'function') {
                  Drupal.behaviors[b].attach(elem);
                }
              }
            }
          });
        }

        if (!authenticated) {
          //Bring up anonymous content
          $(".cacheControlAnonymous").each(function() {
            var id = $(this).attr('id').replace(/-anonymous$/, '');

            if ($("#"+id).length > 0) {
              $("#"+id).replaceWith($(this));
            }
            $(this).attr('id', id);
            $(this).removeClass('cacheControlAnonymous');

          });

          //If needed, process forms with captcha
          if (Drupal.settings.cacheControl.processAnonymousCaptchaForms) {
            $("input[name=captcha_response]").each(function() {
              var form = $(this).parents('form');
              if (!form.hasClass('cacheControlProcessedForm')) {
                form.addClass('cacheControlProcessedForm');
                var formBuildId = form.find('input[name=form_build_id]').val();

                //Add a focus event handler that silently and hopefully unobtrusively rebuilds the form in the background
                form.find('*').focus(function() {
                  //store focused element id so that we can restore focus when the form has been re-rendered
                  var focusedId = $(this).attr("id");

                  $.get(Drupal.settings.basePath+"_cache_control/rebuild_form/"+formBuildId, function(data) {
                    var currentContent = $("#"+focusedId).val();
                    var newForm = $(data);
                    form.replaceWith(newForm);

                    //Some JS components may need initializing
                    for (b in Drupal.behaviors) {
                      if (b != 'cacheControl' && typeof Drupal.behaviors[b].attach == 'function') {
                        Drupal.behaviors[b].attach(newForm);
                      }
                    }

                    //For improved user experience
                    $("#"+focusedId).val(currentContent);
                    $("#"+focusedId).focus();
                  });
                });
              }
            });
          }

          //Further, track statistics for an anonymous user if so requested
          if (cacheNodeId) {
            $.get(Drupal.settings.basePath+"cache_control_statistics_tracker.php?nid="+cacheNodeId);
          }
        }
      }
    }
  }

  function cache_control_debug(str) {
    // console.log(str);
  }
})(jQuery);
