<?php
/**
 * @file
 * Admin functions for cache_control
 *
 * TODO Rewrite the admin interface to follow good Drupal practices
 */

/**
 * Form builder for Cache Control main admin form
 *
 * @see cache_control_admin_submit()
 * @ingroup forms
 */
function cache_control_admin($form, &$form_state) {
  $form = array(
    '#cache' => TRUE,
    '#action' => '',
  );

  $form['fieldset_cache'] = array(
    '#type' => 'fieldset',
    '#title' => t('General settings'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $form['fieldset_cache']['cache_control_enabled'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable Cache Control'),
    '#description' => t('To enable path-specific cache control on the site, check this box. Developers may prefer keeping this disabled while development is on-going.'),
    '#default_value' => variable_get('cache_control_enabled', FALSE),
  );

  $form['fieldset_cache']['cache_control_expires'] = array(
    '#type' => 'textfield',
    '#title' => t('Cache lifetime'),
    '#default_value' => variable_get('cache_control_expires', 5),
    '#size' => 5,
    '#description' => t('The maximum time clients are allowed to cache content from this site.'),
    '#field_suffix' => t('minute(s)'),
  );

  //Disabled until http://drupal.org/node/1267740 is fixed
  /*
  $form['fieldset_cache_statistics'] = array(
    '#type' => 'fieldset',
    '#title' => t('Statistics'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $form['fieldset_cache_statistics']['cache_control_statistic_views'] = array(
    '#type' => 'checkbox',
    '#title' => t('Count content views'),
    '#description' => t('Increment a counter each time content is viewed.'),
    '#default_value' => variable_get('cache_control_statistic_views', FALSE),
  );
  */

  $form['fieldset_cache_misc'] = array(
    '#type' => 'fieldset',
    '#title' => t('Miscellaneous settings'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $form['fieldset_cache_misc']['cache_control_dev_mode'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable development mode'),
    '#description' => t('This makes <i>get_components()</i> wait for one second before returning the content, allowing you to see what your placeholders look like. A debug header is also added notifying why a page load was not cached.'),
    '#default_value' => variable_get('cache_control_dev_mode', FALSE),
  );

  $form['fieldset_cache_misc']['cache_control_use_uid_in_ajax'] = array(
    '#type' => 'checkbox',
    '#title' => t('Use UID in AJAX'),
    '#description' => t('This adds the requesting user\'s uid in the AJAX calls that request components, in addition to using the session id. Check this if you want to log user specific data in your access logs.'),
    '#default_value' => variable_get('cache_control_use_uid_in_ajax', FALSE),
  );


  $form['fieldset_cache_misc']['cache_control_no_cache_content_types'] = array(
    '#type' => 'checkboxes',
    '#title' => t('No-cache content types'),
    '#description' => t('Disable Cache Control for the checked content types. Takes effect when node/% is being cached.'),
    '#default_value' => variable_get('cache_control_no_cache_content_types', array()),
    '#options' => node_type_get_names()
  );

  $form['fieldset_cache_misc']['cache_control_secret'] = array(
    '#type' => 'textfield',
    '#title' => t('Secret'),
    '#description' => t('Value used for session id hash and role combination hash calculations.'),
    '#default_value' => variable_get('cache_control_secret', '0'),
  );


  $form['buttons']['submit'] = array('#type' => 'submit', '#value' => t('Save configuration'));
  $form['buttons']['#weight'] = 1;

  if (!empty($_POST) && form_get_errors()) {
    drupal_set_message(t('The settings have not been saved because of the errors.'), 'error');
  }
  $form['#submit'][] = 'cache_control_admin_submit';
  $form['#theme'] = 'system_settings_form';
  return $form;
}

/**
 * Admin settings form submit handler
 *
 * @see cache_control_admin_submit()
 */
function cache_control_admin_submit($form, &$form_state) {
  $posted_values = $form_state['input'];

  variable_set('cache_control_enabled', $posted_values['cache_control_enabled']);
  variable_set('cache_control_expires', $posted_values['cache_control_expires']);

  if (isset($posted_values['cache_control_statistic_views'])) {
    if ($posted_values['cache_control_statistic_views'] == 1 && !file_exists('cache_control_statistics_tracker.php')) {
      drupal_set_message(t('Please copy file <i>cache_control_statistics_tracker.php</i> to your Drupal installation root before enabling statistics tracking.'), 'warning');
    }
    else {
      variable_set('cache_control_statistic_views', $posted_values['cache_control_statistic_views']);
    }
  }

  variable_set('cache_control_dev_mode', $posted_values['cache_control_dev_mode']);
  variable_set('cache_control_use_uid_in_ajax', $posted_values['cache_control_use_uid_in_ajax']);

  variable_set('cache_control_no_cache_content_types', $posted_values['cache_control_no_cache_content_types']);

  variable_set('cache_control_secret', $posted_values['cache_control_secret']);

  drupal_set_message(t('The configuration options have been saved.'));
}

/**
 * Form builder for making paths available for caching
 *
 * @see cache_control_available_paths_submit()
 * @ingroup forms
 */
function cache_control_available_paths() {
  $form = array();
  $paths = cache_control_get_no_cache_paths();

  $form['available_paths'] = array(
    '#type' => 'checkboxes',
    '#options' => $paths,
    '#title' => t('Paths not currently cached'),
    '#description' => t('For these paths, suitable HTTP headers are sent, disallowing clients to cache the reponses.'),
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => 'Make selected paths cacheable',
  );

  return $form;
}


/**
 * Submit handler
 *
 * @see cache_control_available_paths()
 */
function cache_control_available_paths_submit($form, $form_state) {
  foreach ($form_state['values']['available_paths'] as $path => $add) {
    if ($add) {
      cache_control_set_cacheable_path($path);
    }
  }

  drupal_set_message(t('Changes saved'));
}


/**
 * Form builder for selecting roles for which Cache Control is disabled
 *
 * @see cache_control_roles_submit()
 * @ingroup forms
 */
function cache_control_roles() {
  $form = array();

  $roles = user_roles();
  $role_values = variable_get('cache_control_roles', array());
  $form['roles'] = array(
    '#type' => 'checkboxes',
    '#options' => $roles,
    '#default_value' => $role_values,
    '#title' => t('Disable Cache Control for the following roles'),
    '#description' => t('For these roles, pages are generated normally, not as the anonymous user.'),
  );

  $form['admin'] = array(
    '#type' => 'checkbox',
    '#default_value' => variable_get('cache_control_enable_for_admin_user', FALSE),
    '#title' => t('ENABLE Cache Control for the admin user'),
    '#description' => t('By default, Cache Control is disabled for the Admin user (uid 1). Check this if for some reason you want to enable Cache Control for this user.'),
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => 'Save',
  );

  return $form;
}

/**
 * Submit handler
 *
 * @see cache_control_roles()
 */
function cache_control_roles_submit($form, $form_state) {
  $roles = array();
  foreach ($form_state['values']['roles'] as $role => $selected) {
    if ($selected) {
      $roles[] = $role;
    }
  }
  variable_set('cache_control_roles', $roles);

  variable_set('cache_control_enable_for_admin_user', $form_state['values']['admin']);

  drupal_set_message(t('Changes saved'));
}


/**
 * Gets menu router paths for which caching is currently allowed
 */
function cache_control_get_cacheable_paths() {
  $paths = array();
  $result = db_query("SELECT * FROM {cache_control_allow_cache} ORDER BY path");

  foreach ($result as $row) {
    $paths[] = $row;
  }

  return $paths;
}

/**
 * Gets menu router paths for which caching is not currently enabled
 */
function cache_control_get_no_cache_paths() {
  $paths = array();
  $result = db_query("SELECT path FROM {menu_router} WHERE path NOT IN (SELECT path FROM {cache_control_allow_cache}) AND path NOT LIKE 'admin%' ORDER BY path");

  foreach ($result as $row) {
    $paths[$row->path] = $row->path;
  }

  return $paths;
}

/**
 * Enables caching for a menu router path
 */
function cache_control_set_cacheable_path($path) {
  db_insert('cache_control_allow_cache')->fields(array('path' => $path))->execute();
}

/**
 * Disables caching from a menu router path
 */
function cache_control_unset_cacheable_path($path) {
  db_delete('cache_control_allow_cache')->condition('path', $path, '=')->execute();
}

/**
 * Form builder for currently cached paths listing
 *
 * @see cache_control_define_max_age_paths_submit()
 * @ingroup forms
 */
function cache_control_define_max_age_paths() {
  $form = array();
  //get all paths info
  $paths = cache_control_get_cacheable_paths();

  foreach ($paths as $path) {
    $form['cacheable_path_' . base64_encode($path->path)] = array(
      '#type' => 'checkbox',
      '#title' => $path->path,
    );

    $form['cacheable_path_max_age_' . base64_encode($path->path)] = array(
      '#type' => 'textfield',
      '#value' => ($path->max_age_seconds == NULL ? 0 : $path->max_age_seconds),
      '#size' => 5,
      '#required' => TRUE,
      '#description' => t('Define max age seconds for this cacheable path. Enter 0 to use default cache life time.'),
    );
  }

  $form['submit_remove'] = array(
    '#type' => 'submit',
    '#value' => t('Remove selected paths'),
  );
  $form['submit_save'] = array(
    '#type' => 'submit',
    '#value' => t('Save max age'),
  );

  return $form;
}

/**
 * Submit handler
 *
 * @see cache_control_define_max_age_paths()
 *
 */
function cache_control_define_max_age_paths_submit($form, $form_state) {
  $post_vals = $form_state['input'];
  if ($post_vals['op'] == 'Save max age') {
    $paths = cache_control_get_cacheable_paths();
    foreach ($paths as $path) {
      $max_age = intval($post_vals['cacheable_path_max_age_' . base64_encode($path->path)]);
      db_update('cache_control_allow_cache')
        ->fields(array('max_age_seconds' => $max_age))
        ->condition('path', $path->path, '=')
        ->execute();
    }
    drupal_set_message(t('Changes saved'));
  }
  elseif ($post_vals['op'] == 'Remove selected paths') {
    $paths = cache_control_get_cacheable_paths(TRUE);
    foreach ($paths as $path) {
      $selected = $post_vals['cacheable_path_' . base64_encode($path->path)];
      if ($selected && $selected == 1) {
        cache_control_unset_cacheable_path($path->path);
      }
    }
    drupal_set_message(t('Changes saved'));
  }
}

